#include "qinputdialog.h"
#include <qpainter.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include "HighscoreTable.h"
#include "app_window.h"

#define str(s) xstr(s)
#define xstr(s) #s
#define appName Same V1.2.0


AppWindow::AppWindow( QWidget* parent, const char* name, WFlags f )
	: QMainWindow( parent, name, f )
{

	setCaption(str(appName));
	srand(time(NULL));
        
	machine_id = -1; // Clear machine_id to identify if running on arm / Intel
#ifdef arm
  FILE *fp1 = fopen( "/proc/deviceinfo/product", "r" );
  if ( fp1 ) {
    char linebuf[256];
    fgets( linebuf, 255, fp1 );
    if ( !strncmp( linebuf, "SL-5", 4  ) ) { machine_id = 5;}
    else if ( !strncmp( linebuf, "SL-C", 4  ) ) { machine_id = 7; }
    else {
     machine_id = 5;
     //handle unknown machine like 5x00 series - there 'must' be a 
     // compat mode like on C7x0/C860 series ;-)
     fprintf( stderr, "warning: unrecognized machine: %s : Contact asys3@web.de to \
     support it in next release.\n", linebuf );
    };
    printf("machine %s found !\n", linebuf);
    fclose ( fp1 );
  }
  else 
  {
   printf("NO machine ID found (/proc/deviceinfo/product not exists ! Trying as a 5500.\n");
   machine_id = 5;
  } 
   
#endif  
 if (machine_id == -1){
  QMessageBox *rbox;
  rbox = new QMessageBox("No Zaurus found ...","Resolution = 640x480 ? (No = 320x240)",
                        QMessageBox::Information, QMessageBox::Yes, QMessageBox::No, QMessageBox::NoButton);
 
   if (rbox->exec() == QMessageBox::Yes){
     machine_id = 7;
//  paintEvent(NULL);
   }
 }


	board = new Board(this);
	board->setMachine(machine_id);
//	board->setGeometry(5, 38, 230, 230);
	connect(board, SIGNAL(showScore(int)), SLOT(showScore(int)));
	score = 0;

	m_highscore = new HighscoreTable(10, "Scores");
	config = new Config("Same");
	m_highscore->readScores(config);
        
	highscore = m_highscore->getScore(0);

	connect(board, SIGNAL(finalScore(int)), SLOT(addHighscore(int)));	

	QMenuBar *mb=menuBar();

    gameMenu = new QPopupMenu;
	gameMenu->insertItem( "&New Game", NEW_GAME);
	gameMenu->insertItem( "&Highscores", HIGHSCORE);
	gameMenu->insertItem( "&Clear Highscores", CLEARHIGH);
	gameMenu->insertSeparator();
	gameMenu->insertItem( "&Exit Game", EXIT);
	mb->insertItem("&Game", gameMenu);
	connect( gameMenu, SIGNAL(activated(int)),SLOT(menuSelected(int))); 

    
    settingsMenu = new QPopupMenu;
    tilesMenu    = new QPopupMenu;
    soundMenu    = new QPopupMenu;
        
	tilesMenu->insertItem( "&3D color", TILE1);
        tilesMenu->setItemChecked(TILE1, TRUE);
	tilesMenu->insertItem( "&Basic", TILE3);
	tilesMenu->insertItem( "&Fruit", TILE4);
	tilesMenu->insertItem( "&Puzzle", TILE5);
	connect( tilesMenu, SIGNAL(activated(int)),SLOT(menuSelected(int))); 

        settingsMenu->insertItem( "&Tileset", tilesMenu);
        settingsMenu->insertItem( "&Sound", SOUND);
        settingsMenu->setItemChecked(SOUND, FALSE);
		
	mb->insertItem("&Settings", settingsMenu);
	connect( settingsMenu, SIGNAL(activated(int)),SLOT(menuSelected(int))); 

    helpMenu = new QPopupMenu;
	helpMenu->insertItem( "&About", ABOUT);
	helpMenu->insertItem( "&Rules", RULES);
	mb->insertItem("&Help", helpMenu);
	connect( helpMenu, SIGNAL(activated(int)),SLOT(menuSelected(int))); 

	board->startLevel();
}

void AppWindow::showHighscores(QString name, int score){
   QMessageBox *rbox;
   QString table;
   
table = "<table>";
   
   for (int i=0; i< m_highscore->numEntries(); i++){
      table +=  "<tr><td>";
      if ((score == m_highscore->getScore(i)) && (!QString::compare(name, m_highscore->getName(i))))
        table += "<b>";
      table += QString::number(i+1)+".";
      if ((score == m_highscore->getScore(i)) && (!QString::compare(name, m_highscore->getName(i))))
        table += "</b>";
      table += "</td>";
      table +=  "<td>";
      if ((score == m_highscore->getScore(i)) && (!QString::compare(name, m_highscore->getName(i))))
        table += "<b>";
      table += m_highscore->getName(i);
      if ((score == m_highscore->getScore(i)) && (!QString::compare(name, m_highscore->getName(i))))
        table += "</b>";
      table += "</td>";
      table +=  "<td>";
      if ((score == m_highscore->getScore(i)) && (!QString::compare(name, m_highscore->getName(i))))
        table += "<b>";
      table += QString::number(m_highscore->getScore(i));
      if ((score == m_highscore->getScore(i)) && (!QString::compare(name, m_highscore->getName(i))))
        table += "</b>";
      table += "</td></tr>";

   }      
   
table += "</table>";
   
   rbox = new QMessageBox(this, "Highscores");


if (m_highscore->numEntries())
   rbox->setText(table);
else
   rbox->setText("<center>Sorry, no highscores yet...</center>");
      
   rbox->setCaption("Highscores");
//   rbox->setMaximumWidth(200);
   rbox->exec();
}


void AppWindow::clearHighscores()
{

 QMessageBox *rbox;
 rbox = new QMessageBox("Deleting highscore table","Clear all highscore entries ?",
                        QMessageBox::Warning, QMessageBox::Yes, QMessageBox::Abort, QMessageBox::NoButton);
 
 if (rbox->exec() == QMessageBox::Yes){
 
  delete m_highscore;
  m_highscore = new HighscoreTable(10, "Scores");
  config->clearGroup();
  delete config;
  config = new Config("Same");
  score = 0;
  highscore = 0;
  board->startLevel();
  paintEvent(NULL);
 }
}



void AppWindow::addHighscore(int score)
{
  if(m_highscore->isHighscore(score)){

   //led func. only on Zaurus
#ifdef QT_QWS_EBX
    if (score >= highscore)
      board->setled(1);
    else
      board->setled(0);
#endif
         
    bool ok = FALSE;

   // ask for name
    m_name = QInputDialog::getText("New highscore entry","Please enter name", "nobody", &ok);

   // add score and set led off now
   //led func. only on Zaurus
#ifdef QT_QWS_EBX
   board->offled();
#endif
   if (m_name.isEmpty())
     m_name = QString("nobody");
   m_highscore->addEntry(score, m_name);
   m_highscore->writeScores(config);
   delete config;
   config = new Config("Same");
   
   showHighscores(m_name, score);
  }
}


void AppWindow::showScore(int newScore)
{
	score = newScore;
	if (score > highscore){
          highscore = score;
	  board->setHigh();
	}  
	paintEvent(NULL);
}

void AppWindow::paintEvent ( QPaintEvent * )
{
	QPixmap tpix(width(),height());
	int s=0, offset=0;
	
	if (machine_id == 7) { offset = 0; };
	
	tpix.fill(QColor(199,190,166));
	
	QPainter p(this);
	p.drawPixmap(0,0,tpix);
	p.setFont( QFont("helvetica", 12));

	s = p.fontInfo().pointSize();
//	printf("pointSize %d",s);
	
	char text[30];

	for (int i=1; i<6; i++){
	  p.drawPixmap( 8+(i-1)*16+(i-1)*25+offset, height()-17, board->getTile(i, &s));
	  sprintf(text, ":%d",s);
	  p.drawText(8+(i-1)*16+(i-1)*25+16, height()-5, QString(text));
        }

	if (machine_id == 7) { 
	    // C - models: show score/highscore at the bottom
	    sprintf(text, "Score: %d High: %d", score, highscore);
	    p.drawText(8+(6-1)*16+(6-1)*25+16, height()-5, QString(text));
	}
	else  { 
	    // 5x00 models: show score/highscore at the top
	    sprintf(text, "Score: %d High: %d", score, highscore);
	    p.drawText(8, 34, QString(text));
	}
}


void AppWindow::menuSelected(int id)
{
	switch(id) {
		case EXIT:
			close();
		break;
		case ABOUT:
			QMessageBox *abox;
			abox = new QMessageBox(this, "About");
			abox->setCaption(QString("About ")+QString(str(appName)));
			abox->setText("<center>by <b>Uwe Koch (asys3@web.de)</b><BR>\
			               http://www.linuxvision.de/zaurus.html<BR>\
				       A port of SameGame for AgendaVR3 and after ideas from zTappy<BR>\
				       <b>This is free Software (GPL)</b>");
			abox->exec();
		break;
		case RULES:
			QMessageBox *rbox;
			rbox = new QMessageBox(this, "Rules");
			rbox->setCaption("Same's Rules");
			rbox->setText("<b>Goal:</b> Remove as many adjacent blocks at once. Champions: clear the whole board !<BR>\
				       <b>Removing blocks:</b> Click on a piece if at least one same piece is around \
				             (top, bottom, left or right). Click on one of these pieces, to remove them. \
					     Shrinking of freed place is made from right to left and from bottom up.<BR>\
				       <b>Score:</b>Calculated by the square of (#pieces-2)<BR>\
				       <b>Sound:</b>If you enable key click, every removed piece makes a click.");
			rbox->adjustSize();
			rbox->exec();
		break;
		case TILE1:
		board->setTileset(1);
                settingsMenu->setItemChecked(TILE1, TRUE);
                settingsMenu->setItemChecked(TILE3, FALSE);
                settingsMenu->setItemChecked(TILE4, FALSE);
                settingsMenu->setItemChecked(TILE5, FALSE);
		board->paintEvent(NULL);
		paintEvent(NULL);
		break;
		case TILE3:
		board->setTileset(3);
                settingsMenu->setItemChecked(TILE1, FALSE);
                settingsMenu->setItemChecked(TILE3, TRUE);
                settingsMenu->setItemChecked(TILE4, FALSE);
                settingsMenu->setItemChecked(TILE5, FALSE);
		board->paintEvent(NULL);
		paintEvent(NULL);
		break;
		case TILE4:
		board->setTileset(4);
                settingsMenu->setItemChecked(TILE1, FALSE);
                settingsMenu->setItemChecked(TILE3, FALSE);
                settingsMenu->setItemChecked(TILE4, TRUE);
                settingsMenu->setItemChecked(TILE5, FALSE);
		board->paintEvent(NULL);
		paintEvent(NULL);
		break;
		case TILE5:
		board->setTileset(5);
                settingsMenu->setItemChecked(TILE1, FALSE);
                settingsMenu->setItemChecked(TILE3, FALSE);
                settingsMenu->setItemChecked(TILE4, FALSE);
                settingsMenu->setItemChecked(TILE5, TRUE);
		board->paintEvent(NULL);
		paintEvent(NULL);
		break;
		case NEW_GAME:
		board->startLevel();
		paintEvent(NULL);
		break;
		case HIGHSCORE:
                showHighscores("NO",-1);
		break;
		case CLEARHIGH:
                clearHighscores();
		break;
		case SOUND:
		settingsMenu->setItemChecked(SOUND, !settingsMenu->isItemChecked(SOUND));
		board->setSound(settingsMenu->isItemChecked(SOUND));  
		default:
		break;
	}
}
