#include "board.h"
#include "japxpm.h"
#include "colorxpm.h"
#include "fruitxpm.h"
#include "puzzlexpm.h"
#include <qpainter.h>
#include <qmessagebox.h>

#include <fcntl.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include "sharp_char.h"

//const int xblocks = 14, yblocks = 14, blockTypes = 5;

Board::Board(QWidget* parent, const char* name, WFlags f ): QWidget(parent,name,f)
{
        setSound(FALSE);

//activate buzzer func. only on Zaurus
#ifdef QT_QWS_EBX
  	openbuzz();
	openled();
#endif

        setTileset(1);
}



QPixmap Board::getTile(int tilenumber, int * count){
 *count=tileCount[tilenumber];
 return *blockPic[tilenumber];
};

void Board::setHigh(){
  isHigh=1;
}

int Board::openled(){
  led = open ("/dev/sharp_led", O_WRONLY);
 if(led == -1){
  perror("/dev/sharp_led");
  return 1;
 }
 return 0;
}

int Board::openbuzz(){
 buzz = open("/dev/sharp_buz", O_WRONLY);
 if(buzz == -1){
  perror("/dev/sharp_buz");
  return 1;
 }
 return 0;
}

int Board::setbuzz(){
int stat;
 stat= ioctl(buzz, SHARP_BUZZER_MAKESOUND, 2);
 if(stat == -1){
  perror("buzz ioctl");
  return -1; 
 }
 return 0 ;
}

int Board::setMachine(int machine)
{
 machine_id = machine;
 
 if (machine == 7){
  setGeometry(5, 38, 630, 353);
  xblocks = 39;
  yblocks = 22;
  }
 else{
  setGeometry(5, 38, 230, 230);
  xblocks = 14;
  yblocks = 14;
 } 
//  setGeometry(5, 38, 630, 385);
  blockTypes = 5;  

 return 0 ;
}


int Board::setled(int score){
int stat;

  sharp_led_status st;

  st.which = SHARP_LED_COLLIE_1;
  if (score)
    st.status = LED_COLLIE_1_VFSTBLINK;
  else
    st.status = LED_COLLIE_1_NORMBLINK;
     
  stat = ioctl(led, SHARP_LED_SETSTATUS, &st);

 if(stat == -1){
  perror("led fast ioctl");
  return -1; 
 }
 return 0 ;
}

int Board::offled(){
int stat;

  sharp_led_status st;

  st.which = SHARP_LED_COLLIE_1;
  st.status = LED_COLLIE_1_DEFAULT;

  stat = ioctl(led, SHARP_LED_SETSTATUS, &st);

 if(stat == -1){
  perror("led off ioctl");
  return -1; 
 }
 return 0 ;
}


void Board::setTileset(int tilesetnumber){
   switch (tilesetnumber){
   case 1:
        //3D TileSet color
	blockPic[0] = new QPixmap(p0_xpm);
	blockPic[1] = new QPixmap(block);
	blockPic[2] = new QPixmap(bluegem);
	blockPic[3] = new QPixmap(yellowgem);
	blockPic[4] = new QPixmap(diamond);
	blockPic[5] = new QPixmap(disc);
   break;
   case 2:
        //3D TileSet bw removed
   break;
   case 3:
        //basic TileSet
	blockPic[0] = new QPixmap(j0_xpm);
	blockPic[1] = new QPixmap(j1_xpm);
	blockPic[2] = new QPixmap(j2_xpm);
	blockPic[3] = new QPixmap(j3_xpm);
	blockPic[4] = new QPixmap(j4_xpm);
	blockPic[5] = new QPixmap(j5_xpm);
   break;
   case 4:
        //fruit TileSet
	blockPic[0] = new QPixmap(emptypiece);
	blockPic[1] = new QPixmap(piece10000);
	blockPic[2] = new QPixmap(piece40000);
	blockPic[3] = new QPixmap(piece50000);
	blockPic[4] = new QPixmap(piece60000);
	blockPic[5] = new QPixmap(piece70000);
   break;
   case 5:
        //puzz-le TileSet
	blockPic[0] = new QPixmap(p0_xpm);
	blockPic[1] = new QPixmap(blue_small);
	blockPic[2] = new QPixmap(green_small);
	blockPic[3] = new QPixmap(purple_small);
	blockPic[4] = new QPixmap(red_small);
	blockPic[5] = new QPixmap(yellow_small);
   break;
  	
   };        
           
	blockWidth  = blockPic[0]->width();
	blockHeight = blockPic[0]->height();
   
}

void Board::setSound(bool s)
{
        sound=s;
}


void Board::startLevel()
{
	score = 0;
	isHigh=0;
	for (int i=0; i<6; i++)
	  tileCount[i]=0;

	emit showScore(0);

	for (int x=0; x<xblocks; x++)
		for (int y=0; y<yblocks; y++){
			map[x][y] = rand() % blockTypes +1;
			tileCount[map[x][y]]++;
			}
//	for (int i=0; i<6; i++)
//	  printf("%d\n",tileCount[i]);
	paintEvent(NULL);
}

int Board::isGameOver(){
    int type;
    for (int y = 0; y < yblocks; y++) {
      for (int x = 0; x < xblocks; x++) {
	if ((type = map[x][y]) == 0) {
	  break;
	}
	for (int i = 0; i < 4; i++) {
	  if (type == getType(x, y, i)) {
	    return false;
	  }
	}
      }
    }
    return true;
}

int Board::getType(int x, int y, int dir){
    int tmp_x, tmp_y;
    tmp_x = x + DIRX[dir];
    tmp_y = y + DIRY[dir];
    if (tmp_x < 0 || tmp_x > xblocks-1 || tmp_y < 0 || tmp_y > yblocks-1 ) {
      return 0;
    }
    
    return map[tmp_x][tmp_y];
    
}


void Board::paintEvent (QPaintEvent *pe) 
{
	QPainter p(this);
	p.moveTo(0,0);

	for (int x=0; x<xblocks; x++)
          for (int y=0; y<yblocks; y++)
	      p.drawPixmap(blockWidth*x+3, blockHeight*y+3, *blockPic[map[x][y]]);
	      
}

bool Board::finished()
{
	for (int x=0; x<xblocks; x++)
		for (int y=0; y<yblocks; y++)
			if (map[x][y])
				return false;
	
	return true;
}

void Board::deleteAll()
{
	for (int x=0; x<xblocks; x++)
	  for (int y=0; y<yblocks; y++)
	    map[x][y] = 0;
}

void Board::mergeEmptySpaces(){

// let's merge vertically
	for (int y=0; y<yblocks; y++){
          for (int x=0;x<xblocks-1;x++){
	    if (map[x][y] ==0){ //hole found - let's merge it
	      int swappiece = x+1;
             
	      while ((swappiece <xblocks) && (map[swappiece][y] == 0)) swappiece++; //skip empty places
	      if (swappiece < xblocks){          //if I found a piece (I'm not at the end of a line...)
	        map[x][y]=map[swappiece][y];     // I exchange it with the left most empty place
	        map[swappiece][y]=0;             // and look for other empty pieces...
		
              }	      
	    }
	  }    	  
        }
// now look if we have empty lines and must shift the bottom up.
// must be extra loop - otherwise horizontal merging interacts with vertical merge
	for (int y=0; y<yblocks-1; y++){ //one line less(last line merged by vertical merge)
	  if (map[0][y] == 0){ //found empty line
            int swappiece = y+1;	  
	      while ((swappiece <yblocks) && (map[0][swappiece] == 0)) swappiece++; //skip empty lines
	      if (swappiece < yblocks)           //if I found a piece (it's an non empty line...)
                for ( int x=0; x<xblocks; x++){  //and switch this line with the most uper empty line
    	          map[x][y]=map[x][swappiece];   
	          map[x][swappiece]=0;
		}
	        
	  }
        }
}

int Board::removePieces( int x, int y, int type)
{
        if ((x<0) || (x>xblocks-1) || (y<0) || (y>yblocks-1) || (map[x][y] != type))
	    return 0;  // out of bounds / neighbour with not the right color 
	             // => nothing to remove 
	map[x][y]=0; //same color => clear piece
                     // and recursively look in the neighborhood until everything cleared
        
	int numbRemoved=1;		     
	numbRemoved += removePieces (x,y-1,type);
	numbRemoved += removePieces (x+1,y,type);
	numbRemoved += removePieces (x,y+1,type);
	numbRemoved += removePieces (x-1,y,type);

// activate buzzer func. only on Zaurus
#ifdef QT_QWS_EBX
        if (sound)
           for ( int x=0; x<numbRemoved; x++) setbuzz();
#endif	
	return numbRemoved;
}

void Board::mousePressEvent ( QMouseEvent *e )
{
	int x = (e->x()-3) / blockWidth;
	int y = (e->y()-3) / blockHeight;
	int type = map[x][y];
	
	if (!type) return; //clicked on empty place - do nothing
	
	if ( (((y-1)>=0)      && (map[x][y-1] == type)) || //look up... 
	     (((x+1)<xblocks) && (map[x+1][y] == type)) || //look right...
	     (((y+1)<yblocks) && (map[x][y+1] == type)) || //look down and
	     (((x-1)>=0)      && (map[x-1][y] == type)) ){ //look left from selected piece
	                                              //if there's a similar piece around
	   map[x][y]=0; // neighbour found => so remove selected piece
	                // now: chain reaction => find all similar pieces adjacent to the
			//neighbours...and remove them
	   
	   int numbRemoved=1;
	   		
	   numbRemoved += removePieces (x,y-1,type);
	   numbRemoved += removePieces (x+1,y,type);
	   numbRemoved += removePieces (x,y+1,type);
	   numbRemoved += removePieces (x-1,y,type);
           
	   tileCount[type]-=numbRemoved;
	   	   
	   score += (numbRemoved-2)*(numbRemoved-2);
	   emit showScore(score);
	   // now all are removed => let's shrink empty places
	   
	   mergeEmptySpaces();
        }
	else return;
	 // you clicked a piece and there's no similar piece around => read the
	 // manual :-) -> nothing to do - try another piece	     
	 
	 
	paintEvent(NULL);
	
	if (isGameOver()){
	  QMessageBox *rbox;
	  rbox = new QMessageBox(this, "GameOver");
	  rbox->setCaption("Game over");
	  rbox->setText("<center>Congratulations !<BR>\
	                 <B>No more moves.</B></center>");
	  
	  
	  if (isHigh){
	  rbox->setText("<center>Congratulations !<BR>\
	                 <B>New Highscore !!</B></center>");
   //led func. only on Zaurus
#ifdef QT_QWS_EBX
	  setled(1);
#endif
	  }
	  rbox->setMaximumWidth(210);
	  rbox->exec();
	  
	  emit finalScore(score);

	}
	
}

